#pragma once

/*
 * UsiTwiSlave.h
 *
 * Created: 2012-07-07
 *  Author: Ulli
 *     See: https://github.com/puuu/USIWire/tree/master/src/USI_TWI_Slave
 *          https://github.com/BlackBears/ATtiny2313-I2C-Slave-Test
 *
 * 2011-12-30 Version 1
 * ---------------------------
 * Initiale Version
 *
 * 2013-12-08 Version 2
 * ---------------------------
 * Die erste Version eines USI-Slave-Bibliothek recht einfach aufgebaut.
 * Die neue Version v2 bietet einen deutlich hheren Komfort.
 *
 * 2022-11-04 Version 3
 * ---------------------------
 * Zur Benutzung mit Arduino berarbeitet. Die Arduino-Bibliotheken werden jedoch nicht bentigt.
 * Die nderungen sind in Modifications.txt dokumentiert.
 *  
 * 2022-11-13 Version 4
 * ---------------------------
 * Fehler beim Erkennen der Startbedingung behoben:
 * SCL wurde nicht immer frei gegeben.
*/

/*
* ------------------------------------------------
* --- Pinbelegungen be den einzelnen Prozessoren:
* ------------------------------------------------
ATtiny25, ATtiny45, ATtiny85:
SDA: PB0
SCL: PB2

ATtiny24, ATtiny44, ATtiny84:
SDA: PA6
SCL: PA4

ATtiny2313, ATtiny4313:
SDA: PB5
SCL: PB7

ATtiny26:
SDA: PB0
SCL: PB2

ATtiny261, ATtiny461, ATtiny861:
SDA: PB0
SCL: PB2

ATmega165, ATmega325, ATmega3250, ATmega645, ATmega6450,
ATmega329, ATmega3290, ATmega169:
SDA: PE5
SCL: PE4


------------------------------------------------- */

// Die folgenden Definitionen werden in 'UsiTwiSlave.cfg.h' bentigt und mssen deshalb vor dem #include stehen!
// -------------------------------------------------------------------------------------------------------------
#define TWI_MODE_STANDARD -1 // Keine Automatismen, funktioniert ausschlielich ber Callback
#define TWI_MODE_SIMPLE   -2 // Ein-Byte-IO
#define TWI_MODE_REGISTER -3 // Register-IO

#define FALSE 0 // fr den Precompiler
#define TRUE 1
#define NONE -1

#include "UrsUsiTwiSlave.cfg.h"

enum class TwiTransferType {
   TwiRead = 0,    // Der Master will Daten von diesem Slave empfangen
   TwiWrite = 1,    // Der Master sendet Daten an diesen Slave
#if (TWI_ACCEPT_BROADCAST_MSG)
   BroadcastRead = 2,  // Der Master will Daten von allen Slaves empfangen
   BroadcastWrite = 3, // Der Master sendet Daten an alle Slaves
#endif // ACCEPT_BROADCAST_MSG
};

// Gibt an, ob der Tranfer ber die Broadcast-Adresse erfolgt.
inline bool TwiIsBroadcast(TwiTransferType rw) {
   return (uint8_t)rw & 2;
}

/*****************************************************************************
Felder & Callback-Funktionen
******************************************************************************/

// --- Modus TWI_MODE_STANDARD -----------------------------------------------------------------------------
#if (TWI_OPERATION_MODE == TWI_MODE_STANDARD)
uint8_t TwiDataRequest(void);        // Die Applikation stellt die Funktionen bereit.
void TwiDataReceived(uint8_t data);
void TwiTransmissionStart(TwiTransferType rw);
void TwiTransmissionStop();
#endif // TWI_OPERATION_MODE == TWI_MODE_STANDARD

// --- Modus TWI_MODE_SIMPLE -------------------------------------------------------------------------------
#if (TWI_OPERATION_MODE == TWI_MODE_SIMPLE)
extern volatile uint8_t TwiData;  // Daten werden ber Variable 'TwiData' ausgetauscht.
extern volatile bool TwiDataReceivedFlag;
static inline void TwiTransmissionStart(TwiTransferType rw) {} // Wird nicht bentigt.
static inline void TwiTransmissionStop() {}

#endif // TWI_OPERATION_MODE == TWI_MODE_SIMPLE

// --- Modus TWI_MODE_REGISTER -----------------------------------------------------------------------------
#if (TWI_OPERATION_MODE == TWI_MODE_REGISTER)
void TwiTransmissionStart(TwiTransferType rw);  // Die Applikation stellt die Funktion bereit.
void TwiTransmissionStop();                     // Die Applikation stellt die Funktion bereit.

// index der mit data beschrieben werden soll.
// Liefert true, wenn das Byte bereits ausgewertet wurde
// und nicht ins Register geschrieben werden soll.
bool TwiDataReceived(uint8_t index, uint8_t data);

bool TwiDataRequest(uint8_t index, uint8_t* pData);

extern volatile I2CRegister_t I2CRegister;  // Daten werden ber Variable 'I2CRegister' ausgetauscht.
#endif // TWI_OPERATION_MODE == TWI_MODE_REGISTER


// --- Adress-Modus ----------------------------------------------------------------------------------------

#if (TWI_FIXED_TWI_ADDRESS != NONE) // Fest vorgegebene Bus-Adresse
void TwiInitialize(void);
#else   // Variable Bus-Adresse
void TwiInitialize(uint8_t TwiAddress);
void TwiSetAddress(uint8_t TwiAddress);
#endif //FIXED_TWI_ADDRESS
